package PHP::File;

use strict;
use warnings;
use Carp 'croak';
use constant FILE_APPEND => 1;
use Fcntl qw(:DEFAULT) ;
use FindBin qw($Bin);
use File::Spec;
use Digest::MD5 qw(md5_hex);
use LWP::Simple qw(get);
our(@ISA,@EXPORT,@EXPORT_OK,$VERSION);
require Exporter;
@ISA = qw(Exporter);

@EXPORT = qw(file_get_contents FILE_APPEND file_put_contents);
@EXPORT_OK = qw(file_get_contents FILE_APPEND file_put_contents parse_ini_file tempnam);

$VERSION = 1.01;

sub Version {
	return $VERSION;
}

sub file_get_contents {
	my $sys_temp_file = undef;
	if ($_[0] =~ /^(?:ht|f)tps?:\/\//i) {
		$sys_temp_file=get($_[0]);
		croak 'Cannot get the page' unless defined $sys_temp_file;
	} else {
		my $temp_name = File::Spec->canonpath($_[0]) || $_[0];
		sysopen(SYS_READ_TEMP_FILE,$temp_name,O_RDONLY) or croak 'Cannot open the file ' . $temp_name;
		eval{flock(SYS_READ_TEMP_FILE,1);};
		if (-B $temp_name) {
			eval{binmode(SYS_READ_TEMP_FILE);};
		}
		croak $! unless defined sysread SYS_READ_TEMP_FILE,$sys_temp_file,-s $temp_name;
		close(SYS_READ_TEMP_FILE);
	}
	return $sys_temp_file;
}

sub file_put_contents {
	$_[1] ||='';
	my $bytes = undef;
	unless ($_[0]) {
		croak 'Please give a file name';
	}
	if ($_[0] =~ /^(?:ht|f)tps?:\/\//i) {
		croak 'We cannot provide support to write a remote file';
	} else {
	my $chmod = 0;
	my $temp_name=File::Spec->canonpath($_[0]) || $_[0];
	$chmod = 1 if !-e $temp_name or -d $temp_name;
	sysopen(SYS_CHANGE_TEMP_FILE,$temp_name,O_WRONLY | ($_[2]? O_APPEND  : O_TRUNC)  | O_CREAT) or croak 'Cannot write the file ' . $temp_name;
	eval{flock(SYS_CHANGE_TEMP_FILE,2);};
	if ($_[1] !~ /\n|\012/) { # perhaps?
		eval{binmode(SYS_CHANGE_TEMP_FILE);};
	}
	$bytes = syswrite SYS_CHANGE_TEMP_FILE,$_[1],length($_[1]);
	close(SYS_CHANGE_TEMP_FILE);
	chmod(0644,$temp_name) if $chmod;
	}
	return $bytes
}

sub parse_ini_file {
	my (%H,$stack)=((),'');
	$_[1] ||=0;
	my $temp_name=File::Spec->canonpath($_[0]) || $_[0];
	open(SYS_INI_FILE ,$temp_name) or croak 'Cant open ini file ' . $_[0];
	while (<SYS_INI_FILE>) {
		chop;
		next if /^\s*$|^\s*;.*|[^=[]/;
		if ($_[1] && $_ =~ /^\[([^]]*)\]$/) {$H{$1}='';$stack=$1;next;}
		if ($_ =~ /^([^=]+)=(?:\s*"([^"]*)"|\s*'([^']*)'|([^;]*))/) {if (!$_[1]) {$H{$stack}=+{$1=>$2,%{$H{$stack}}};}else{$H{$1}=[$2,@{$H{$1}}];}}
	}
	close(SYS_INI_FILE);
	return %H;
}

sub tempnam {
	my ($DIRECTORY,$PREFIX)=@_;
	my ($file,$test);
	$DIRECTORY ||= File::Spec->catdir($Bin . '/');
	$PREFIX ||= '';
	eval {	local $SIG{ALRM} = sub {die('timeout')};
		alarm 15;
		LOOP: { do {
			$file = $PREFIX.md5_hex($$,time(),rand(999999999));
			$test = File::Spec->catfile($DIRECTORY,$file);
			unless (-e $test) {
				croak 'Can not create a temporary file' unless defined file_put_contents($test);
				last LOOP;
			}
		} while (1); }
		alarm 0;};
	if ($@) { croak 'The time of create file was expired, please try again' unless $@ eq 'timeout'; }
	return $file;
}

1;

__END__;


=file_get_contents('./foo')

Read a local o remote file

=file_put_contents('./foo.bar',['.. perl is a language ..',[append]])

Create or append an existing file, for append use FILE_APPEND constant

=parse_ini_file('./foo.ini',[expand]) not exporter by default

parse a ini file just like PHP or try to make the same :-), expand is a boolean that active or not the expand reference array or hash 

=tempnam(directory, [prefix]) not exporter by default

create a file in the directory and return the name, put some prefix name file if you want

don't forget unlink($tempnamreturned); to delete this file.

Copyright Eial 2003 <eiial@hotmail.com>
