package PHP::OB;

use strict;
use warnings;
use Carp 'croak';
our(@HEADER,$DONTREPEAT,$VERSION,$ERASE_NAME,$DONOTCHANGE,$ERASE_VALUE,$ERASE_ACTIVE,$HEADERSENT,$ADD_ACTIVE,$ADD_NAME,$ADD_VALUE,$ACTIVE,$BUFFER,$NOBUFFER,@ISA,@EXPORT,@EXPORT_OK,%EXPORT_TAGS);
require Exporter;

@ISA=qw(Exporter);

@EXPORT=qw(ob_start output_add_rewrite_var ob_end_flush);
@EXPORT_OK=qw(headers_list setcookie output_reset_rewrite_vars headers_sent header ob_get_clean ob_get_flush ob_end_clean ob_start reset_header output_add_rewrite_var ob_get_contents ob_get_length ob_end_flush ob_end ob_flush ob_clean);
%EXPORT_TAGS=(standart=>[qw(header setcookie ob_start output_add_rewrite_var ob_end ob_end_flush ob_flush)],head=>[qw(header headers_list headers_sent reset_header setcookie ob_start ob_end_flush ob_end ob_flush)],ob=>[qw(ob_get_clean ob_get_flush ob_end_clean ob_start ob_get_contents ob_get_length ob_end_flush ob_end ob_flush ob_clean)]);

$VERSION = 1.03;

$BUFFER="";

# Public function

sub Version {
	return $VERSION;
}

sub ob_start {
	unless ($ACTIVE) {
		&_ob_start;
		$ACTIVE=1;
	}
}

sub ob_end {
	if ($ACTIVE) {
		$ACTIVE=0;
		&_ob_end;
	}
}

sub ob_clean {
	if ($ACTIVE) {
		$BUFFER = "";
	}
}

sub reset_header {
	@HEADER=();
}

sub header {
	my $header=_trim(shift @_);
	my $check = $header =~ /:\s+/;
	if (!$HEADERSENT & $check) {
		if ($header=~/^content-type:|^location:/i) {
			unless ($DONTREPEAT) {
				$DONTREPEAT=(&_nph & $header=~/location/i)?1:(&_nph?2:3);
				push @HEADER, $header;
			}
		} else {
			unshift @HEADER,$header;
		}
	}
	else {
		croak $check?'All headers was sent':'Wrong header: '.$header;
	}
}

sub setcookie {
	my $cookie=shift;
	if (UNIVERSAL::isa($cookie,'CGI::Cookie')) {
		$cookie=$cookie->as_string;
		if ($cookie) {
			header('Set-Cookie: '.$cookie);
		}
	} else {
		croak 'Wrong cookie value, see CGI::Cookie to know how create new cookies.';
	}
}

sub headers_list {
	return @HEADER;
}

sub headers_sent {
	return $HEADERSENT;
}

sub ob_get_contents {
	return $ACTIVE ? $BUFFER : undef;
}

sub ob_get_length {
	return $ACTIVE ? length($BUFFER) : undef;
}

sub ob_get_clean {
	if ($ACTIVE) {
		my $temp = &ob_get_contents;
		&ob_clean;
		return $temp;
	}
}

sub ob_flush {
	if ($ACTIVE) {
		&_ob_end;
		if (scalar @HEADER > 0) {
			my $protocol = $ENV{SERVER_PROTOCOL} || 'HTTP/1.0';
			if ($DONTREPEAT==1) {
				unshift @HEADER, $protocol.' 302 MOVED';
			} elsif ($DONTREPEAT==2) {
				unshift @HEADER, $protocol.' 200 OK';
			}
			print join &_crlf,@HEADER;
			print &_crlf x 2;
			&reset_header; $HEADERSENT=1;
		}
		if ($ADD_ACTIVE) {
			&_output_add_rewrite_var;
		}
		if ($ERASE_ACTIVE) {
			&_output_reset_rewrite_vars;
		}
		print $BUFFER; #i'm tired and i don't why bizarre copy of HASH
		&ob_clean;
		&_ob_start;
	}
}

sub ob_end_clean {
	if ($ACTIVE) {
		&ob_clean;
		&ob_end;
	}
}

sub ob_end_flush {
	if ($ACTIVE) {
		&ob_flush;
		&ob_end;
	}
}

sub ob_get_flush {
	if ($ACTIVE) {
		my $temp = &ob_get_contents;
		&ob_end;
		&ob_clean;
		return $temp;
	}
}

sub output_add_rewrite_var {
	unless ($DONOTCHANGE) {
		$ADD_NAME=$_[0];
		$ADD_VALUE=$_[1];
		$ADD_ACTIVE=1;
	}
}

 
sub output_reset_rewrite_vars {
	$ERASE_NAME=$_[0];
	$ERASE_VALUE=$_[1];
	$ERASE_ACTIVE=1;
}

sub force_dont_change {
	unless ($DONOTCHANGE) {
		$DONOTCHANGE=1;
		return 1;
	} else {
		return undef;
	}
}

# Private functions

sub _ob_end {
	close STDOUT;
	open STDOUT, '>&', $NOBUFFER or croak 'Cannot open the old output'.$!;
	close $NOBUFFER;
	select STDOUT;
}

sub _ob_start {
	open $NOBUFFER, '>&', STDOUT or croak 'Cannot save the actual output '.$!;
	close STDOUT;
	open STDOUT, '>', \$BUFFER or croak 'Cannot start the buffer '.$!;
	select STDOUT;
}

sub _output_add_rewrite_var {
	$BUFFER =~ s#(<form[^>]*>)#$1.'<input type="hidden" name="'.$ADD_NAME.'" value="'.$ADD_VALUE.'" />'#gmei;
	$BUFFER =~ s#(<(?:a|frame|iframe) [^>]*(?:href|src)\s*=\s*)(?:("[^"]*"|'[^']*'|[^> ]*))([^>]*>)#$1._add_var($2).$3#geim;
}

sub _output_reset_rewrite_vars {
	$BUFFER =~ s#(<form[^>]*>)\s*<input type="hidden" name="${ADD_NAME}" value="${ADD_VALUE}" />#$1#gmi;
	$BUFFER =~ s#(<(?:a|frame|iframe) [^>]*(?:href|src)\s*=\s*)(?:("[^"]*"|'[^']*'|[^> ]*))([^>]*>)#$1._erase_var($2).$3#geim;
}

sub _add_var {
	my $url=shift;
	if ($url =~ /\?/) {
		$url =~ s#([^?]*)\?(.*)#$1.'?'.$ADD_NAME.'='.$ADD_VALUE.($2?'&'.$2:$2)#me;
	} elsif ($url =~ /^["']/) {
		$url =~ s#(["'])(.*?)(['"])#$1.$2.'?'.$ADD_NAME.'='.$ADD_VALUE.$3#me;
	} else {
		$url=_trim($url).'?'.$ADD_NAME.'='.$ADD_VALUE;
	}
	return $url;
}

sub _erase_var {
	my $url=shift;
	if ($url =~ /\?$ERASE_NAME=$ERASE_VALUE&/) {
		$url =~ s#$ERASE_NAME=$ERASE_VALUE&##;
	} else {
		$url =~ s#\?$ERASE_NAME=$ERASE_VALUE##;
	}
	return $url;
}

sub _crlf {
	return ($^O =~ /VMS/i) ? "\n" : (("\t" ne "\011") ? "\r\n" : "\015\012");
}

sub _nph {
	my $nph = $ENV{SCRIPT_NAME};
	$nph=~ s#^.*/(.*)$#$1#sm;
	return $nph =~ /^nph-/i ? 1 : 0;
}

sub _trim {
	my $value=shift;
	$value =~ s#^\s*(.*?)\s*$#$1#sm;
	return $value;
}

sub END {
	if ($ACTIVE) {
		&ob_flush;
		&ob_end;
	}
}

1;

__END__;


=ob_start (import by default)

Turn on output buffering. While output buffering is active no output is sent from the script, instead the output is stored in an internal buffer. 

=ob_end_flush (import by default)

Flush (send) the output buffer and turn off output buffering.

=ob_get_contents

Return the output buffer or undef in other case.

=ob_get_length

Return the length of the output buffer or undef in other case

=ob_end_clean

Clean (erase) the output buffer and turn off output buffering

=ob_flush

Flush (send) the output buffer

=ob_get_flush

Flush the output buffer, return it as a string and turn off output buffering.

=ob_get_clean

Flush the output buffer, return it as a string and clean (erase) the output buffer.

=output_add_rewrite_var(name,value) (import by default)

This function rewrite the URLs and forms with the given variable.

=output_reset_rewrite_vars(name,value)

This function resets the URL rewriter and undo the changes made by output_add_rewrite_var().

=ob_end

Turn off output buffering

=header(header)

Send a HTTP headers

=setcookie(value)

Send a Cookie, value must be (see CGI::Cookie):

use CGI::Cookie;

my $value = new CGI::Cookie(-name=>$name,-value=>$value,-expires=>$expires,-path=>$path,-domain=>$domain,-secure=>$secure);

setcookie($value);

=reset_header

Clean all headers

=headers_list

return all headers (sent with header function), if not send yet.

=headers_sent

check if headers was sent, return 1 or 0

=Here is a list of the function sets you can import

:standart, import header setcookie ob_start output_add_rewrite_var ob_end ob_end_flush ob_flush.

:head, import header headers_list headers_sent reset_header setcookie ob_start ob_end_flush ob_end ob_flush

:ob, import ob_get_clean ob_get_flush ob_end_clean ob_start ob_get_contents ob_get_length ob_end_flush ob_end ob_flush ob_clean

The ob_start output_add_rewrite_var ob_end_flush functions will be importer by default. if you need anyone else, import this directly.

Ej:

#!/usr/bin/perl

use PHP::OB qw(ob_start header);

ob_start();

print 'hello world !!!'."\n";

header('Content-Type: text/html');

The script response:

....

Content-type: text/html

hello World !!!

...


Ej2:

#!/usr/bin/perl

use PHP::OB qw(:standart);

print 'Content-Type: text/html'."\n\n";

print 'no buffer here'."\n";

ob_start();

print 'buffering'."\n";

output_add_rewrite_var('name','value');

print '<form><a href="script.php">Script</a></form>'."\n";

ob_flush();

print 'buffering'."\n";

ob_end_flush();

print 'no buffer here'."\n";

The script response:

....

Content-type: text/html

no buffer here
buffering
<form><input type="hidden" name="name" value="value" /><a href="script.php?name=value">Script</a></form>
buffering
no buffer here

...

Copyright Eial 2005 <eiial@hotmail.com>
