package PHP::Session;

use strict;
use warnings;
use Carp 'croak';
our(@ISA,@EXPORT,@EXPORT_OK,$VERSION,%SES,$SID,%param,$serialize,$rewrite,$name,$cookie,$refresh,$handle,$type,$expiration,%cookieparam);
require Exporter;
require Tie::Hash;
@ISA = qw(Tie::Hash Exporter);
@EXPORT = qw(session_start session_use_trans_sid %SES $SID SID);
@EXPORT_OK = qw(session_start session_destroy session_get_cookie_params session_save_handler_param session_save_path session_use_cookies session_use_trans_sid session_save_handler SID $SID %SES session_name session_set_cookie_params session_cache_expire session_unset session_refresh);

$VERSION = 1.02;

$name = 'id';
$expiration = 900;
$refresh=0;
$type='file';
$cookie=1;
%param=(-path=>'./session');
$rewrite=0;
$SID=0;

# Public functions

sub Version {
        return $VERSION;
}

sub session_start {
        if ($type=~/^mysql$/i) {
                $handle=eval {require PHP::Session::MySql;
                new PHP::Session::MySql(\%param,$expiration); };
        } elsif ($type=~/^file$/i) {
                $handle=eval{require PHP::Session::File;
                new PHP::Session::File(\%param,$expiration);};
        } else {
                croak 'You must select some type of session handle';
        }
        unless ($handle) {
                croak 'Your session handle do not exists, perhaps you lost some module, change the session handle and try again or '.$@;
        }
        $handle->clean;
        tie %SES,'PHP::Session::_tie',$name,$handle,$cookie,$refresh,$rewrite,\%cookieparam;
        $SID=$PHP::Session::_tie::sid;
}

sub session_name {
        my $nsession=shift;
        if ($nsession) {
                $name=$nsession;
        }
        return $name;
}

sub session_cache_expire {
        my $cesession=shift;
        if ($cesession!~/[^0-9]/) {
                $expiration=$cesession;
        }
        return $expiration;
}

sub session_refresh {
        my $rsession=shift;
        if ($rsession=~/[0-1]/) {
                $refresh=$rsession;
        }
        return $refresh;
}

sub session_save_handler {
        my $shsession=shift;
        if ($shsession =~ /^mysql$|^file$/i) {
                $type=$shsession;
        }
        return $type;
}

sub session_use_trans_sid  {
        my $urlsession=shift;
        if ($urlsession =~ /[0-1]/i) {
                $rewrite=$urlsession;
                $cookie=$rewrite?0:$cookie;
        }
        return $rewrite;
}

sub session_use_cookies {
        my $ucsession=shift;
        if ($ucsession =~ /[0-1]/i) {
                $cookie=$ucsession;
                $rewrite=$cookie?0:$rewrite;
        }
        return $cookie;
}

sub session_save_handler_param {
        my %data=@_;
        if (scalar %data >0) {
                %param=%data;
        }
        return %param;
}

sub session_save_path {
        my $path=shift;
        if ($path) {
                $param{-path}=$path;
        }
        return $path;
}

sub session_set_cookie_params {
        my ($expire,$path,$domain,$secure)=@_;
        %cookieparam=(expire=>$expire,path=>$path,domain=>$domain,secure=>$secure);
}

sub session_destroy {
        $handle->destroy($name);
        undef(%SES); # becouse a %SES reference still exist
}

sub session_unset {
        undef(%SES);
}

sub session_get_cookie_params {
        return scalar %cookieparam > 0?%cookieparam:undef;
}

sub SID {
        if ($SID) {
                return $name.'='.$SID;
        } else {
                return undef;
        }
}

package PHP::Session::_tie;

use strict;
use warnings;
use Carp 'croak';
use PHP::OB qw(ob_start output_add_rewrite_var ob_flush headers_sent setcookie);
use CGI qw(param);
use CGI::Cookie;
our($VERSION,$name,$handle,$cookie,$rewrite,$refresh,$store,$sid,$cookieparam);


sub TIEHASH {
        my $self = shift;
        ($name,$handle,$cookie,$refresh,$rewrite,$cookieparam) = @_;
        if (!$cookie) {
                $sid=param($name);
        } else {
                my %cookies = fetch CGI::Cookie;
                if ($cookies{$name}) {
                        $sid=$cookies{$name}->value;
                }
        }
        $store = bless {SESSION=>{}}, $self;
        ($store,$sid)=$handle->load($store,$sid);
        if (!$cookie & $rewrite) {
                ob_start();
                output_add_rewrite_var($name,$sid);
                unless (PHP::OB::force_dont_change()) {
                        croak 'You already set do not change rewrite variables, Session module can not continue with transparent sid active';
                }
        } elsif ($cookie) {
                my $cookie = new CGI::Cookie(-name => $name,-value => $sid,-expires =>$cookieparam->{expire},-domain=>$cookieparam->{domain},-path =>$cookieparam->{path},-secure => $cookieparam->{secure});
                if ($cookie) {
                        if ($PHP::OB::ACTIVE) {
                                unless (headers_sent()) {
                                        setcookie($cookie);
                                } else {
                                        croak 'Becareful, you already sent all headers, you can not send a session cookie, try to flush after call session_start';
                                }
                        } else {
                                print 'Set-Cookie: '.$cookie->as_string.PHP::OB::_crlf;
                        }
                }
        }
        return $store;
}

sub FETCH {
        my ($self,$key) = @_;
        return $self->{SESSION}{$key};
}

sub STORE {
        my ($self,$key,$value) = @_;
        $self->{SESSION}{$key} = $value;
        $handle->save($self,$sid) if $refresh;
}

sub DELETE {
        my ($self,$key)  = @_;
        delete $self->{SESSION}{$key};
        $handle->save($self,$sid) if $refresh;
}

sub EXISTS {
    my ($self,$key)  = @_;
    exists $self->{SESSION}{$key} ;
}

sub CLEAR {
        my $self = shift;
        foreach  (keys %{$self->{SESSION}}) {
                $self->DELETE($_);
        }
}

sub FIRSTKEY {
        my $self = shift;
        my $a = keys %{$self->{SESSION}};
        each(%{$self->{SESSION}});
}

sub NEXTKEY {
        my $self = shift;
        return each(%{$self->{SESSION}});
}

sub SCALAR {
        my $self = shift;
        return scalar(%{$self->{SESSION}});
}

sub DESTROY {
        my $self = shift;
        if ($handle) {
                $handle->save($self,$sid) unless $refresh;
        }
        if (!$cookie && $rewrite && $PHP::OB::ACTIVE) {
                ob_flush();
        }
}

sub END {
        if ($store) {
                $handle->save($store,$sid) unless $refresh;
        }
        $handle->finish if $handle;

}

1;
__END__;

=session_start

start a session

=session_destroy

destroy a session

=session_get_cookie_params

return a hash with all cookie param if exists or undef

=session_save_handler_param

set all param to the selected handle,

        -file need -path by default is /session,
        -myqsl need -database, -host, -user, -pass, -table, [-mysql4 if you support innoDB and you want use this]

if use database you must be create some of this tables:

if use mysql 4 +, or if you have mysql 3.23 with innoDB active

CREATE TABLE `session`(
        sid VARCHAR(50) DEFAULT NULL,
        data BLOB,
        time TIMESTAMP
) TYPE= InnoDB;

please active -mysql4=>1

else

CREATE TABLE `session`(
        sid VARCHAR(50) DEFAULT NULL,
        data BLOB,
        time TIMESTAMP
) TYPE= MyISAM;

=session_save_path

change or return the actual path when use file handle, default /session

=session_use_cookies

set if you want use or not cookie, default true

=session_use_trans_sid

set if you want use transparent insertion id, default false

=session_save_handler

set with handle must be use the session module, for now support two modules, mysql and file, default file

=$SID

it's have a session id

=SID

return a string with session name and session id join by a equal

=%SES

the magic variable, you can use just like a normal hash.

=session_name

return or change the session name, default id

=session_set_cookie_params(expire,path,domain,secure)

set the cookie param, to jump expire or secure use 0, to jump path or domain use ''

=session_cache_expire

set the limit of time that stay alive (if not have any access) the session, default 900 (15 min)

=session_unset

clean the session

=session_refresh

set 1 if you want that the module register any change with each access or 0 to regiter all change when the script finish, default 0 it's faster

=exporter by default:

session_start session_use_trans_sid %SES $SID SID

don't forget exporter always %SES at least.

Ej1:

use PHP::Session;

print 'Content-type: text/html'."\n\n";

session_use_trans_sid(1);
session_start();

$SES{hello} = 'hello world!!!'

print 'Form: <form><input type="submit"></form><br/>'."\n";
print 'Link: <a href="script.pl">script</a>'."\n";


exit;

The script response:

....

Content-Type: text/html

Form: <form><input type="hidden" name="id" value="6a888b16556bb24f0e6f608dd5085baa" /><input type="submit"></form><br/>
Link: <a href="script.pl?id=6a888b16556bb24f0e6f608dd5085baa">script</a>

....

Ej2:

use PHP::Session;

print 'Content-type: text/html'."\n\n";

session_start();

$SES{hello} = 'hello world!!!'

print 'Form: <form><input type="submit"></form><br/>'."\n";
print 'Link: <a href="script.pl">script</a>'."\n";


exit;

The script response:

....

Content-Type: text/html
Set-Cookie: id=6a888b16556bb24f0e6f608dd5085baa; path=/;

Form: <form><input type="submit"></form><br/>
Link: <a href="script.pl">script</a>

....

Ej3:

use PHP::Session qw(session_start session_save_path %SES);

print 'Content-type: text/html'."\n\n";
session_save_path('./tmp'); # change the directory session
session_start();

$SES{hello} = 'hello world!!!'

exit;


Copyright Eial 2005 <eiial@hotmail.com>
