package PHP::Session::File;

use strict;
use warnings;
use Carp 'croak';
use PHP::File qw(file_get_contents file_put_contents tempnam);
use PHP::Serialize;
use File::Spec;
use IO::Dir;

our $VERSION=1.00;

sub Version {
	return $VERSION;
}

sub new {
	my ($class,$data,$expiration)=@_;
	return bless {
				path=>$data->{-path},
				expiration=>$expiration,
			},$class;
}

sub save {
	my ($class,$self,$sid) = @_;
	my $file = File::Spec->catfile($class->{path},$sid);
	file_put_contents($file,serialize(\%{$self->{SESSION}}));
}

sub load {
	my ($class,$self,$sid) = @_;
	my $file = File::Spec->catfile($class->{path},$sid);
	if ($file && -e $file && !-d $file) {
		$self->{SESSION}={%{unserialize(file_get_contents($file))}};
	} else {
		$self->{SESSION}={};
		$sid=tempnam($class->{path});
		file_put_contents(File::Spec->catfile($class->{path},$sid),serialize(\%{$self->{SESSION}}));
	}
	return ($self,$sid);
}

sub clean {
	my $class=shift;
	my $pid = fork();
	unless ($pid) {
		my $directory = IO::Dir->new($class->{path}) or croak 'Cant open session directory for clean';
		while (defined($_ = $directory->read)) { 
			next if /^\./; 
			my $file = File::Spec->catfile($class->{path},$_); 
			unlink($file) if (((stat($file))[9] + $class->{expiration}) <= time());
		}
		exit(0);
	}
}

sub destroy {
	my ($class,$name)=@_;
	unlink File::Spec->catfile($class->{path},$name);
}

sub finish {}

1;

__END__;

=comment

internal module for PHP::Session

Copyright Eial 2005 <eiial@hotmail.com>
